# coding: utf-8

import os
import re
import glob
import ssl
import datetime
import socket

import pyquery
import click
import requests
from requests.auth import HTTPBasicAuth
from jmb.tools.conf import read
from urllib3.util import parse_url

from jmb import tools

BUFFER_DAYS = 25

@click.command('domain')
@click.option('--ssl/--no-ssl', is_flag=True, help="Test/Don't test https. Default: test both", default=None)
@click.option('--www/--no-www', is_flag=True, help="Test/Don't prepend www", default=True)
@click.option('--user', '-u', help="The user for basic authentication")
@click.option('--password', '-p', help="The pass for basic authentication")
@click.option('--permutate', '-x', help="Permutate on all domains in string eg: -x 'it com org'")
@click.option('--verbose', '-v', help="Be verbose and declare the hostname you're testing")
@click.option('--follow-redirects', '-f', is_flag=True, default=False, help="Follow all redirects to get the real page")
@click.argument('domains', nargs=-1)
def test_response(ssl, www, user, password, domains, permutate, verbose, follow_redirects):
    """Test response from servers by connecting. Show title or http_code and Location.
    Arguments may be domains or pages, possibly with protocol. As in:

      jmb test domain acomea.it

      jmb test domain https://acomea.it/faicomeilfai
    """
    if ssl is None:
        protocols = ['http', 'https']
    elif not ssl:
        protocols = ['http']
    else:
        protocols = ['https']

    hostnames = get_hostnames(domains, permutate, www)


    for hostname in hostnames:
        if hostname.startswith('http'):
            url = hostname
            test_url(url, user, password, verbose, follow_redirects=follow_redirects)
        else:
            for proto in protocols:
                url = u"{proto}://{hostname}".format(**locals())
                test_url(url, user, password, verbose, follow_redirects=follow_redirects)


def get_hostnames(domains, permutate='', www=True):
    if isinstance(domains, basestring):
        domains = [domains]
    if www:
        ## cogema.thux.it does not require adding www. This make it impossible to
        hostnames =  list(domains) + ['www.%s' % domain for domain in domains
                                      if len(domain.split('.')) < 3]

    else:
        hostnames = domains
    if permutate:
        hostnames = ['{}.{}'.format(host, ext) for host in hostnames for ext in permutate.split()]
    return hostnames


def test_url(url, user=None, password=None, verbose=False, follow_redirects=False):
    """Test an url, possibly using user/pass and possibly following redirects
    """
    if verbose:
        click.secho("*** %s" % url, fg='magenta')
    if user:
        auth = HTTPBasicAuth(user, password)
    else:
        auth = None
    try:
        resp = requests.get(url, auth=auth, allow_redirects=follow_redirects)
    except requests.exceptions.ConnectionError:
        click.secho(u"{}: connection FAILED".format(url), fg='red')
        return

    if resp.status_code >= 300 and resp.status_code < 400:
        Location = resp.headers['Location']
        msg = u"{url} => {resp.status_code}  ({Location})".format(**locals())
        click.secho(msg)
    elif resp.status_code >= 200 and resp.status_code < 300:
        d = pyquery.PyQuery(resp.content)
        try:
            title = (d('title').contents()[0]).strip()
        except IndexError:
            title = "NO TITLE"

        msg = u"{url} => {resp.status_code} ({title})".format(**locals())
        click.echo(click.style(msg, fg='green'))
    else:
        msg = u"{url} ==> {resp.status_code} {resp.reason}".format(url=url, resp=resp)
        click.echo(click.style(msg, fg='red'))

# https://serverlesscode.com/post/ssl-expiration-alerts-with-lambda/

@click.command('expiry')
@click.option('--number', '-n', is_flag=True, help="Return just number", default=False)
@click.option('--www/--no-www', is_flag=True, help="Test/Don't prepend www", default=False)
@click.option('--permutate', '-x', help="Permutate on all domains in string eg: -x 'it com org'")
@click.option('--buffer-days', '-d', help="max days to issue a warning", default=BUFFER_DAYS)
#@click.option('--follow-redirects', '-f', is_flag=True, default=False, help="Follow all redirects to get the real page")
@click.argument('domains', nargs=-1)
def ssl_expires_in(domains, buffer_days, permutate, www, number):
    """Check hostname certificate expiry by connecting to destinations. """

    domains = get_hostnames(domains, permutate, www)
    for domain in domains:

        remaining = read.ssl_valid_time_remaining(domain)
        show_expiry(domain, remaining, number=number, buffer_days=buffer_days)


def show_expiry(domain, remaining, number=False, buffer_days=None):
    # if the cert expires in less than two weeks, we should reissue it
    if remaining == 'Expired':
        # cert has already expired - uhoh!
        if number:
            click.echo('-1')
        else:
            click.secho(u"{}: Already expired".format(domain), fg='red')
        #raise AlreadyExpired("Cert expired %s days ago" % remaining.days)
    elif remaining < datetime.timedelta(days=buffer_days):
        # expires sooner than the buffer
        if number:
            click.echo(remaining.days)
        else:
            click.secho(u"{}: {} needs refresh".format(domain, remaining.days), fg='yellow')
    else:
        # everything is fine
        if number:
            click.echo(remaining.days)
        else:
            click.secho(u"{}: {} OK".format(domain, remaining.days), fg='green')
        #return False

@click.command('cert')
@click.option('--domain', '-d', is_flag=False, help="args are interpreted as domains (i.e.: must have a certificate already existent with that name)", default=False)
@click.option('--enabled', '-e', is_flag=True, help="look for certificates in nginx enabled ",
              default=False)
@click.option('--number', '-n', is_flag=True, help="Return just number", default=False)
@click.option('--buffer-days', '-D', help="max days to issue a warning", default=BUFFER_DAYS)
@click.argument('nginx_conf_files', nargs=-1)
def get_expiry_from_cert(nginx_conf_files, domain, number, buffer_days, enabled):
    "Read expiry from file"

    if nginx_conf_files and enabled:
        msg = u"Opt enabled implies all certificates from sites-enabled!!! no args permitted"
        raise click.BadParameter(msg)

    if domain and enabled:
        msg = u"Opt domain already searches domain from args, incompatible with 'enabled'"
        raise click.BadParameter(msg)

    if enabled:
        certificates = list(get_enabled_certificates())

    if domain:
        certificates = []
        domains = certificates
        for domain in domains:
            certificates += '{}/live/{}/cert.pem'.format(letsencrypt_conf, domain)

    if nginx_conf_files:
        certificates = list(get_enabled_certificates(files=nginx_conf_files))

    for cert_file in certificates:
        domain, expiry, remaining, alternative_names = read.read_certificate(cert_file)
        show_expiry(domain, remaining, number, buffer_days)
        click.secho(u"    => {}".format(alternative_names), fg='blue')

def get_enabled_certificates(nginx_conf=None, files=None):
    "get all certificates declared in nginx enabled configurations"

    nginx_conf = nginx_conf or tools.NGINX_ENABLED_DIR
    for conf in files or glob.glob(os.path.join(nginx_conf, '*')):
        with open(conf) as f:
            for line in f:
                m = re.match('\s*ssl_certificate\s+(?P<cert>[-_\d\w\./]+)', line)
                if m:
                    yield m.group('cert')
